using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

namespace VeeRifter {
	public class VAMcam : MVRScript {
		
		private WebCamTexture webcam;
		private WebCamDevice[] devices;
		private FreeControllerV3 controller;
		private GameObject webcamScreen;
		private Renderer screenRenderer;
		private List<string> deviceNameList;
		private List<string> reqHeightList;	
		private List<string> reqWidthList;
		private List<string> reqFPSList;
		private JSONStorableStringChooser deviceChoice;
		private JSONStorableStringChooser heightChoice;
		private JSONStorableStringChooser widthChoice;
		private JSONStorableStringChooser FPSChoice;
		private JSONStorableString About;
		private JSONStorableString camResInfo;
		private JSONStorableFloat screenScale;
		private JSONStorableBool flipVertical;
		private JSONStorableBool flipHorizontal;
		private JSONStorableBool Playing;
		private UIDynamicButton camStart;
		private UIDynamicButton camStop;
		private float AspectRatio;
		private bool doneInit = false;
		private bool Enabled = false;

		public override void Init(){
			try{
				if (containingAtom.type != "Empty")
				{
					SuperController.LogError($"Plugin must be used with an 'Empty' atom, not a '{containingAtom.type}'");
					return;
				}				
				webcam = new WebCamTexture();
				devices = WebCamTexture.devices;
				deviceNameList = new List<string>();
				deviceNameList.Add("Select");
				for (int i = 0; i < devices.Length; i++)
				{
					deviceNameList.Add(devices[i].name);
				}

				reqWidthList = new List<string>();
				reqWidthList.Add("Select");
				reqWidthList.Add("1920");
				reqWidthList.Add("1280");
				reqWidthList.Add("1024");
				reqWidthList.Add("960");
				reqWidthList.Add("640");

				reqHeightList = new List<string>();
				reqHeightList.Add("Select");
				reqHeightList.Add("1080");
				reqHeightList.Add("960");
				reqHeightList.Add("720");
				reqHeightList.Add("576");
				reqHeightList.Add("480");
				reqHeightList.Add("360");

				reqFPSList = new List<string>();
				reqFPSList.Add("Select");
				reqFPSList.Add("60");
				reqFPSList.Add("30");
				reqFPSList.Add("15");

				About = new JSONStorableString("About", "<b><color=brown>VAMcam</color></b>\n by VeeRifter\n15th May 23");
				UIDynamicTextField dtfAbout = CreateTextField(About, false);
				dtfAbout.UItext.alignment = TextAnchor.MiddleCenter;
				dtfAbout.UItext.supportRichText = true;
				dtfAbout.height = 20;

				camResInfo = new JSONStorableString("Webcam Resolution Info", "\nWebcam Inactive");
				UIDynamicTextField dtfcamResInfo = CreateTextField(camResInfo, true);
				dtfcamResInfo.UItext.alignment = TextAnchor.MiddleCenter;
				dtfcamResInfo.UItext.supportRichText = true;
				dtfcamResInfo.height = 20;

				deviceChoice = new JSONStorableStringChooser("Webcam Device", deviceNameList, "Select", "Webcam Device", selectDevice);
				CreateScrollablePopup(deviceChoice, false).popupPanelHeight = 340f;
				deviceChoice.storeType = JSONStorableParam.StoreType.Full;
				RegisterStringChooser(deviceChoice);
				deviceChoice.popup.labelTextColor = Color.blue;

				FPSChoice = new JSONStorableStringChooser("Requested FPS", reqFPSList, "Select", "Requested FPS", selectFPS);
				CreateScrollablePopup(FPSChoice, true).popupPanelHeight = 280f;
				FPSChoice.storeType = JSONStorableParam.StoreType.Full;
				RegisterStringChooser(FPSChoice);
				FPSChoice.popup.labelTextColor = Color.blue;

				widthChoice = new JSONStorableStringChooser("Requested Width", reqWidthList, "Select", "Requested Width", selectWidth);
				CreateScrollablePopup(widthChoice, false).popupPanelHeight = 400f;
				widthChoice.storeType = JSONStorableParam.StoreType.Full;
				RegisterStringChooser(widthChoice);
				widthChoice.popup.labelTextColor = Color.blue;

				heightChoice = new JSONStorableStringChooser("Requested Height", reqHeightList, "Select", "Requested Height", selectHeight);
				CreateScrollablePopup(heightChoice, true).popupPanelHeight = 460f;
				heightChoice.storeType = JSONStorableParam.StoreType.Full;
				RegisterStringChooser(heightChoice);
				heightChoice.popup.labelTextColor = Color.blue;

				screenScale = new JSONStorableFloat("Screen Scale", 1.0f, SetScale, 0.1f, 10.0f, true, true);
				screenScale.storeType = JSONStorableParam.StoreType.Full;
				RegisterFloat(screenScale);
				CreateSlider(screenScale, false);

				flipVertical = new JSONStorableBool("Flip Vertical", false,FlipV);
				flipVertical.storeType = JSONStorableParam.StoreType.Full;
				RegisterBool(flipVertical);
				CreateToggle(flipVertical, true);

				flipHorizontal = new JSONStorableBool("Flip Horizontal", false,FlipH);
				flipHorizontal.storeType = JSONStorableParam.StoreType.Full;
				RegisterBool(flipHorizontal);
				CreateToggle(flipHorizontal, true);

				UIDynamic spacerLeft = CreateSpacer(false);
				spacerLeft.height = 1f;

				UIDynamic spacerRight = CreateSpacer(true);
				spacerRight.height = 6f;

				camStart = CreateButton("Start Webcam", false);
				camStart.buttonColor = Color.green;
				camStart.buttonText.color = Color.black;
				camStart.button.onClick.AddListener(webcamStart);

				camStop = CreateButton("Stop Webcam", true);
				camStop.buttonColor = Color.red;
				camStop.buttonText.color = Color.white;
				camStop.button.onClick.AddListener(webcamStop);		

				Playing = new JSONStorableBool("Playing", false);
				Playing.storeType = JSONStorableParam.StoreType.Full;
				RegisterBool(Playing);

				SuperController.singleton.onSceneLoadedHandlers += OnSceneLoaded;
				createScreen();
				doneInit = true;
				}
			catch (Exception e){
				SuperController.LogError("Exception caught in Init(): " + e);
			}
		}

		void OnSceneLoaded()
		{
			if (Playing.val && doneInit)
				webcamStart();
		}
		
		void FlipV(bool flip)
		{
			float y = webcamScreen.transform.localScale.y;
			float x = webcamScreen.transform.localScale.x;
			float z = webcamScreen.transform.localScale.z;
			webcamScreen.transform.localScale = new Vector3(x, y, -z);
		}		

		void FlipH(bool flip)
		{
			float y = webcamScreen.transform.localScale.y;
			float x = webcamScreen.transform.localScale.x;
			float z = webcamScreen.transform.localScale.z;
			webcamScreen.transform.localScale = new Vector3(-x, y, z);
		}

		void SetScale(float scale)
		{
			float x = webcamScreen.transform.localScale.x;
			float y = webcamScreen.transform.localScale.y;
			float z = webcamScreen.transform.localScale.z;
			float Vflip = flipVertical.val ? -1.0f : 1.0f;
			float Hflip = flipHorizontal.val ? -1.0f : 1.0f;
			webcamScreen.transform.localScale = new Vector3(Hflip * scale * AspectRatio, y, Vflip * scale);
		}

		void webcamStart()
		{
			if(Enabled && !webcam.isPlaying)
			{
				createScreen();
				webcam.Play();
				if(webcam.isPlaying)
				{
					Playing.val = true;
					camResInfo.val = "\nActual Cam Resolution : " + Convert.ToString(webcam.width) + " x " + Convert.ToString(webcam.height);
					AspectRatio = (float)webcam.width/(float)webcam.height;
					SetScale(screenScale.val);
				}
				else
				{
					SuperController.LogError("Webcam device failed to start !");
				}
			}
		}

		void webcamStop()
		{
			if(webcam.isPlaying)
			{
				webcam.Stop();
				Playing.val = false;
				Destroy(webcamScreen);
				webcamScreen = null;
				camResInfo.val = "\nWebcam Inactive";
			}				
		}
		
		void createScreen()
		{
			if(webcamScreen == null)
			{
				controller = containingAtom.mainController;
				webcamScreen = GameObject.CreatePrimitive(PrimitiveType.Plane);
				webcamScreen.transform.Rotate(90.0f, 0.0f, 0.0f, Space.Self);//Rotate screen from horizontal to vertical position
				Destroy(webcamScreen.GetComponent<Collider>());// Remove the collider
				webcamScreen.transform.SetParent(controller.gameObject.transform, false); //Parent to 'Empty' atom
				resizeMesh(webcamScreen);//Reduce the initial size
				screenRenderer = webcamScreen.GetComponent<Renderer>();
				screenRenderer.material = new Material(Shader.Find("Sprites/Default"));
				screenRenderer.material.mainTexture = webcam;
			}
		}

		void resizeMesh(GameObject go)
		{
			Mesh mesh = go.GetComponent<MeshFilter>().mesh;
			Vector3[] verts = mesh.vertices;
			for (int i = 0; i < verts.Length; i++)
			{  // reduce size of all vertices by a factor of 10
				verts[i].x = verts[i].x / 10.0f;
				verts[i].y = verts[i].y / 10.0f;
				verts[i].z = verts[i].z / 10.0f;
			}
			mesh.vertices = verts;
			mesh.RecalculateBounds();
		}

		private void selectDevice(string selected)
		{
			if (selected == "Select")
				return;
			webcam.deviceName = selected;
		}

		private void selectHeight(string selected)
		{
			if (selected == "Select")
				return;
			webcam.requestedHeight = Int16.Parse(selected);
		}

		private void selectWidth(string selected)
		{
			if (selected == "Select")
				return;
			webcam.requestedWidth = Int16.Parse(selected);
		}

		private void selectFPS(string selected)
		{
			if (selected == "Select")
				return;
			webcam.requestedFPS = Int16.Parse(selected);
		}

		void OnEnable()
		{
			if(doneInit)
			{
				createScreen();
				SetScale(screenScale.val);
				if(Playing.val)// if wecam was playing prior to plugin being disabled
					webcam.Play();//then resume playing immediately on re-enable
			}
			Enabled = true;
		}

		void OnDisable()
		{
			Enabled = false;
			if(webcam.isPlaying)
			{
				Playing.val = true;
				webcam.Stop();
			}
			if(webcamScreen != null)
			{
				Destroy(webcamScreen);
				webcamScreen = null;
			}
		}
		
		void OnDestroy()
		{
			SuperController.singleton.onSceneLoadedHandlers -= OnSceneLoaded;
			OnDisable();
		}
	}
}